@@ -90,7 +90,7 @@ static void free_snapshot_data(struct ploop_snapshot_data *data)
 }
 
 int ploop_add_snapshot_entry(struct ploop_disk_images_data *di, const char *guid,
-		const char *parent_guid)
+		const char *parent_guid, int temporary)
 {
 	struct ploop_snapshot_data **tmp;
 	struct ploop_snapshot_data *data;
@@ -121,6 +121,7 @@ int ploop_add_snapshot_entry(struct ploop_disk_images_data *di, const char *guid
 	di->snapshots = tmp;
 	data->guid = strdup(guid);
 	data->parent_guid = strdup(parent_guid);
+	data->temporary = temporary;
 
 	if (data->guid == NULL || data->parent_guid == NULL) {
 		ploop_err(ENOMEM, "strdup failed");
@@ -150,7 +151,7 @@ int ploop_di_add_image(struct ploop_disk_images_data *di, const char *fname,
 		return ret;
 	}
 
-	ret = ploop_add_snapshot_entry(di, guid, parent_guid);
+	ret = ploop_add_snapshot_entry(di, guid, parent_guid, 0);
 	if (ret) {
 		free(top_guid);
 		return ret;
@@ -181,6 +182,15 @@ void ploop_di_change_guid(struct ploop_disk_images_data *di, const char *guid, c
 		strcpy(di->top_guid, new_guid);
 }
 
+void ploop_di_set_temporary(struct ploop_disk_images_data *di, const char *guid)
+{
+	int i;
+
+	i = find_snapshot_by_guid(di, guid);
+	if (i != -1)
+		di->snapshots[i]->temporary = 1;
+}
+
 struct ploop_disk_images_data *alloc_diskdescriptor(void)
 {
 	struct ploop_disk_images_data *p;
@@ -202,27 +212,95 @@ struct ploop_disk_images_data *alloc_diskdescriptor(void)
 	return p;
 }
 
-void ploop_free_diskdescriptor(struct ploop_disk_images_data *di)
+void ploop_clear_dd(struct ploop_disk_images_data *di)
 {
 	int i;
 
-	if (di == NULL)
-		return;
-
 	for (i = 0; i < di->nimages; i++)
 		free_image_data(di->images[i]);
+
+	free(di->images);
+	di->images = NULL;
+	di->nimages = 0;
+
 	for (i = 0; i < di->nsnapshots; i++)
 		free_snapshot_data(di->snapshots[i]);
 
-	free(di->images);
 	free(di->snapshots);
+	di->snapshots = NULL;
+	di->nsnapshots = 0;
+
 	free(di->top_guid);
+	di->top_guid = NULL;
+}
+
+void ploop_close_dd(struct ploop_disk_images_data *di)
+{
+	if (di == NULL)
+		return;
+
+	ploop_clear_dd(di);
+
 	free(di->runtime->xml_fname);
 	free(di->runtime->component_name);
 	free(di->runtime);
+
 	free(di);
 }
 
+void ploop_free_diskdescriptor(struct ploop_disk_images_data *di)
+{
+	return ploop_close_dd(di);
+}
+
+/* Lock and read DiskDescriptor.xml
+ * The ploop_open_dd() should be used to get ploop_disk_images_data
+ */
+int ploop_lock_dd(struct ploop_disk_images_data *di)
+{
+	int ret;
+
+	if (!di || !di->runtime || !di->runtime->xml_fname) {
+		ploop_err(0, "Unable to lock: DiskDescriptor.xml is not opened");
+		return -1;
+	}
+
+	ret = ploop_lock_di(di);
+	if (ret)
+		return ret;
+
+	/* Update the DiskDescriptor.xml representation after lock */
+	if (ploop_read_dd(di)) {
+		ploop_unlock_di(di);
+		return -1;
+	}
+
+	return 0;
+}
+
+int ploop_open_dd(struct ploop_disk_images_data **di, const char *fname)
+{
+	char *path;
+	struct ploop_disk_images_data *p;
+
+	path = realpath(fname, NULL);
+	if (path == NULL) {
+		ploop_err(errno, "Can't resolve %s", fname);
+		return SYSEXIT_DISKDESCR;
+	}
+
+	p = alloc_diskdescriptor();
+	if (p == NULL) {
+		free(path);
+		return SYSEXIT_MALLOC;
+	}
+
+	p->runtime->xml_fname = path;
+	*di = p;
+
+	return 0;
+}
+
 int find_image_idx_by_guid(struct ploop_disk_images_data *di, const char *guid)
 {
 	int i;
@@ -404,9 +482,13 @@ int ploop_di_merge_image(struct ploop_disk_images_data *di, const char *guid, ch
 	/* Caller passed child_guid S2 to delete S1 (S1 <- S2 <- S3) (S2 <- S3)
 	 * so it has merge S2 to S1 and we should update all S1 referrences to S2
 	 */
-	for (i = 0; i < di->nsnapshots; i++)
-		if (guidcmp(di->snapshots[i]->guid, snapshot->parent_guid) == 0)
+	for (i = 0; i < di->nsnapshots; i++) {
+		if (guidcmp(di->snapshots[i]->guid, snapshot->parent_guid) == 0) {
 			strcpy(di->snapshots[i]->guid, guid);
+			/* preserve temporary flag */
+			di->snapshots[i]->temporary = snapshot->temporary;
+		}
+	}
 	for (i = 0; i < di->nimages; i++)
 		if (guidcmp(di->images[i]->guid, snapshot->parent_guid) == 0)
 			strcpy(di->images[i]->guid, guid);
