@@ -383,7 +383,7 @@ void ploop_unlock(int *lckfd);
 // fs util
 int get_partition_device_name(const char *device, char *out, int size);
 int make_fs(const char *device, const char *fstype, unsigned int fsblocksize);
-void tune_fs(const char *target, const char *device, unsigned long long size);
+void tune_fs(int balloonfd, const char *device, unsigned long long size);
 int resize_fs(const char *device, off_t blocks);
 int dumpe2fs(const char *device, struct dump2fs_data *data);
 int e2fsck(const char *device, int flags, int *rc);
@@ -406,7 +406,7 @@ PL_EXT int read_line(const char *path, char *nbuf, int len);
 int read_line_quiet(const char *path, char *nbuf, int len);
 int is_valid_blocksize(__u32 blocksize);
 int run_prg(char *const argv[]);
-int run_prg_rc(char *const argv[], int *rc);
+int run_prg_rc(char *const argv[], int close_std_mask, int *rc);
 int p_memalign(void **memptr, size_t alignment, size_t size);
 PL_EXT int guidcmp(const char *p1, const char *p2);
 int auto_mount_image(struct ploop_disk_images_data *di,
@@ -429,5 +429,7 @@ PL_EXT int ploop_change_fmt_version(struct ploop_disk_images_data *di,
 		int new_version, int flags);
 int ploop_get_dev_by_delta(const char *delta, const char *topdelta,
 		const char *component_name, char **out[]);
-
+int check_snapshot_mount(struct ploop_disk_images_data *di,
+		int temporary, const char *parent_fname,
+		const char *child_fname, const char *child_guid);
 #endif
